<?php
## Gladius Database Engine
# @author legolas558
# @version 0.1
# Licensed under GNU General Public License (GPL)
#
#
# ADODB driver

class gladius_driver_ADOConnection extends ADOConnection
{
	var $nameQuote = '`';
	var $sysDate = 'DATE()';
	var $sysTimeStamp = 'NOW()';
	
	var $g;	// instance of Gladius engine

	function gladius_driver_ADOConnection()
	{
		$this->g = new Gladius();
		global $ADODB_FETCH_MODE;
		$this->g->fetch_mode = $ADODB_FETCH_MODE;
		$this->dbtype = 'gladius';
	}

	/**
	 * Connection to database server and selected database
	 *
	 * @access private
	 */

	function _connect($host_ignored, $username_ignored, $password_ignored, $database = "", $persistent_ignored, $forcenew_ignored)
	{
		if (!empty($database))
			return $this->SelectDB( $database );
		else
			$this->database = $database;
		return true;
	}

	/**
	 * Choose a database to connect.
	 *
	 * @param dbname 	is the name of the database to select
	 * @return 			true or false
	 * @access public
	 */

	function SelectDB($dbname)
	{
		$this->database = $dbname;

		return $this->g->SelectDB($dbname);
	}

	/**
	 * Return database error message
	 * Usage: $errormessage =& $db->ErrorMsg();
	 *
	 * @access public
	 */

	function ErrorMsg()
	{
		return $this->g->errstr;
	}

	/**
	 * Return database error number
	 * Usage: $errorbo =& $db->ErrorNo();
	 *
	 * @access public
	 */

	function ErrorNo()
	{
		return $this->g->errno;
	}

	/**
	 * Returns # of affected rows from insert/delete/update query
	 *
	 * @access public
	 * @return integer Affected rows
	 */

	function Affected_Rows()
	{
		return $this->g->affected_rows;
	}

	/**
	 * Returns the last record id of an inserted item
	 * Usage: $db->Insert_ID();
	 *
	 * @access public
	 */

	function Insert_ID()
	{
		return $this->g->insert_id;
	}

	/**
	 * Correctly quotes a string so that all strings are escape coded.
	 * An example is  $db->qstr("Haven't a clue.");
	 *
	 * @param string			the string to quote
	 * @param [magic_quotes]	if $s is GET/POST var, set to get_magic_quotes_gpc().
	 *
	 * @return  single-quoted string IE: 'Haven\'t a clue.'
	 */

	function qstr($string, $magic_quotes=false)
	{
		return "'".$this->g->escape($string, $magic_quotes)."'";
	}

	function QMagic($string)
	{
		return $this->qstr($string, get_magic_quotes_gpc());
	}

	/**
	 * Returns concatenated string
	 * Usage: $db->Concat($str1,$str2);
	 *
	 * @return concatenated string
	 * NOT CHECKED
	 */
	function Concat()
	{
		$arr = func_get_args();
		$list = implode(', ', $arr);

		if (strlen($list) > 0) return "CONCAT($list)";
		else return '';
	}

	//* NOT CHECKED
	function IfNull( $field, $ifNull )
	{
		return " IFNULL($field, $ifNull) ";
	}

	/**
	 * Closes database connection
	 * Usage: $db->close();
	 *
	 * @access public
	 */

	function Close()
	{
		// ?
	}

	 /**
	 * Returns All Records in an array
	 *
	 * Usage: $db->GetAll($sql);
	 * @access public
	 */

	function &GetAll($sql, $inputarr = false)
	{
		$data =& $this->GetArray($sql, $inputarr);
		return $data;
	}

	/**
	 * Returns All Records in an array
	 *
	 * Usage: $db->GetArray($sql);
	 * @access public
	 */

	function &GetArray($sql, $inputarr = false)
	{
		$data = $this->Execute($sql, $inputarr);
		if (!is_bool($data))
			$data = $data->GetArray();
		return $data;
	}

	/**
	 * Executes SQL query and instantiates resultset methods
	 *
	 * @access private
	 * @return mixed Resultset methods
	 * NOT CHECKED
	 */

	function &do_query( $sql, $offset, $nrows, $inputarr=false )
	{
		$false = false;

		$limit = '';
		if ($nrows == -1) {
			if ($offset != -1)
				$limit = ' LIMIT '.$offset;
		} else
 			$limit = ' LIMIT '.$offset.', '.$nrows;

		if ($inputarr && is_array($inputarr)) {
			$sqlarr = explode('?', $sql);
			if (!is_array(reset($inputarr))) $inputarr = array($inputarr);
			foreach($inputarr as $arr) {
				$sql = ''; $i = 0;
				foreach($arr as $v) {
					$sql .= $sqlarr[$i];
					switch(gettype($v)){
						case 'string':
							$sql .= $this->qstr($v);
							break;
						case 'double':
							$sql .= str_replace(',', '.', $v);
							break;
						case 'boolean':
							$sql .= $v ? 1 : 0;
							break;
						default:
							if ($v === null)
								$sql .= 'NULL';
							else $sql .= $v;
					}
					$i += 1;
				}
				$sql .= $sqlarr[$i];
				if ($i+1 != sizeof($sqlarr))
					return $false;
				$this->sql = $sql . $limit;
				$time_start = array_sum(explode(' ', microtime()));
				$this->query_count++;
				$recordset = $this->g->Query( $this->sql );
				$this->query_time_total += (array_sum(explode(' ', microtime())) - $time_start);
				if($this->debug)
				{
					$this->outp($sql . $limit);
				}
				if ($recordset === false) { // error handling if query fails
					if ($fn = $this->raiseErrorFn)
						$fn($this->dbtype, 'EXECUTE', $this->ErrorNo(), $this->ErrorMsg(), $this->sql, $inputarr, $this);
					return $false;
				}
			}
		}
		else
		{
				$this->sql = $sql . $limit;
				$time_start = array_sum(explode(' ', microtime()));
				$this->query_count++;
				$recordset = $this->g->query( $this->sql );
				$this->query_time_total += (array_sum(explode(' ', microtime())) - $time_start);
				if($this->debug)
					$this->outp($sql . $limit);
		}

		if ($recordset === false) { // error handling if query fails
			if ($fn = $this->raiseErrorFn)
				$fn($this->dbtype, 'EXECUTE', $this->ErrorNo(), $this->ErrorMsg(), $this->sql, $inputarr, $this);
			return $false;
		}

		if ($recordset === true) { // return simplified recordset for inserts/updates/deletes with lower overhead
			$rs = new ADORecordSet_empty();
			return $rs;
		}

//		$recordset->_fetch();	// unnecessary

		return $recordset;
	}

}

class gladius_driver_ResultSet extends Gladius_Resultset { }

?>