<?php
## Gladius Database Driver for ADODB lite
# @author legolas558
# Licensed under GNU General Public License (GPL)
#
#
# Operator classes

class begin_expr extends Gladius_Core {

var $bool_op;
var $next;

	## used in chains
	function evaluate() {
		return $this->test();
	}

	function _evaluate() {
		return true;
	}

	function test() {
		global $flagg;
		if (@$flagg) {
			echo ' (<b>'.get_class($this).'</b> [';
		}

		$self = $this->_evaluate();
		global $flagg;
		if (@$flagg)
			echo ']= '.($self ? 'true' : 'false').' )';

		if (!isset($this->next)) {
			global $flagg;
			if (@$flagg)
				echo '<br />';
			return $self;
		}

		global $flagg;
		if (@$flagg)
			echo ' '.$this->bool_op;

		switch ($this->bool_op) {
			case 'or':
				return ($self || $this->next->test());
			case 'and':
				return ($self && $this->next->test());
		}
		$this->_error(_G__UNKNOWN_BOOL_OP, $this->bool_op);
		return null;
	}
}

function sql_regex($m) {
	if (strlen($m[0])==3) {
		if (($m[0]{0}=='\\') and
			($m[0]{1}=='\\')
		)
			return '%';
		return $m[0]{0}.$m[0]{1}.'.*?';
	}
	return '.*?';
}

class expr extends begin_expr {

var $op;
var $left;
var $right;


	function expr($left, $right, $op) {
		$this->left = $left;
		$this->right = $right;
		$this->op = $op;
	}

	function _evaluate() {
		$lh = $this->left->evaluate();
		$rh = $this->right->evaluate();

		global $flagg;
		if (@$flagg) {
			echo $lh.' '.$this->op.' '.$rh;
		}

		//TODO: enforce correct automatic type casting as SQL92 states

/*		// automatic casting, is it ok for SQL92?
		if (is_null($lh)) {
			switch(gettype($rh)) {
				case 'string':
					$lh = '';
					break;
				case 'double':
					$lh = (double)0;
					break;
				case 'integer':
					$lh = (integer)0;
					break;
			}
		}
		if (is_null($rh)) {
			switch(gettype($lh)) {
				case 'string':
					$rh = '';
					break;
				case 'double':
					$rh = (double)0;
					break;
				case 'integer':
					$rh = (integer)0;
					break;
			}
		}

		if (gettype($rh) !== gettype($lh)) {
			$this->_error(_G__TYPE_MISMATCH, gettype($rh), gettype($lh));
			return null;
		}	*/

/*// for debugging
		global $flagg;
		if (@$flagg) {

		echo '<b>EXPR</b> (('.get_class($this->left);
		if (get_class($this->left)=='field')
			echo ' "'.$this->left->content->name.'" ) '.$this->left->content->value;
		else
			echo ') '.$this->left->content;
		echo ' '.$this->op.' ('.get_class($this->right);
		if (get_class($this->right)=='field')
			echo ' "'.$this->right->content->name.'" ) '.$this->right->content->value;
		else
			echo ') '.$this->right->content;
		}
		/// .*/

		switch ($this->op) {
			case '=':
				return equ_comp($lh, $rh);
			case '<>':
				return !equ_comp($lh, $rh);
			case '>=':
				if (gettype($lh) == 'string') {
					return (strpos($lh, $rh)===0);
				}
				return ($lh >= $rh);
			case '<=':
				if (gettype($lh) == 'string') {
					return (strpos($lh, $rh)===strlen($lh)-strlen($rh));
				}
				return ($lh <= $rh);
			case '>':
				if (gettype($lh) == 'string') {
					return (strpos($lh, $rh)===0);
				}
				return ($lh > $rh);
			case '<':
				if (gettype($lh) == 'string') {
					return (strpos($lh, $rh)===strlen($lh)-strlen($rh));
				}
				return ($lh < $rh);
			case 'nlike':
				return !$this->_like_eval($lh, $rh);
			case 'like':
				return $this->_like_eval($lh, $rh);
			default:
				die('<br>GRTE: "'.$this->op.'" not handled');
		}
	}

	function _like_eval($lh, $rh) {
		if (gettype($lh) != 'string') {
			$this->_error(_G__TYPE_MISMATCH, gettype($rh), gettype($lh));
			return null;
		}

		$rh = '/'.preg_replace_callback('/.?.?%/', 'sql_regex', preg_quote($rh)).'$/iA';
		return preg_match($rh, $lh);
	}

}

class in_op extends begin_expr {
	var $values;
	var $hand;

	function in_op(&$hand, &$values) {
		$this->hand =& $hand;
		$this->values =& $values;
	}

	function _evaluate() {
		$val = $this->hand->evaluate();
		return in_array($this->hand->evaluate(), $this->values);
	}

}

class nin_op extends in_op {

	function nin_op(&$hand, &$values) {
		$this->hand =& $hand;
		$this->values =& $values;
	}

	function _evaluate() {
		return !parent::_evaluate();
	}

}

class arith_expr extends begin_expr {
	var $op;
	var $left;
	var $right;

	function arith_expr($l, $r, $op) {
		$this->left = $l;
		$this->right = $r;
		$this->op = $op;
	}

	function test() {
		die('virtual method called');
	}

	function evaluate() {
		$lv = $this->left->evaluate();
		$rv = $this->right->evaluate();
		$bf = debug_backtrace();
		switch ($this->op) {
			case '+':
				return $lv + $rv;
			case '-':
				return $lv - $rv;
			case '*':
				return $lv * $rv;
			case '/':
				return $lv / $v;
			case '%':
				return $lv % $v;
		}
	}
}

class value extends Gladius_Core {

var $content;

	function value($content) {
		$this->content = $content;
	}

	function evaluate() {
		return $this->content;
	}
}

class field extends value {

	function field(&$fld) {
		$this->content =& $fld;
	}

	function evaluate() {
		return $this->content->value;
	}
}

class func extends value {
	var $name;
	var $argument;

	function func($fn, &$arg) {
		$this->name = $fn;
		$this->argument =& $arg;
	}

	function evaluate() {
		die('GRTE: functions not yet supported');
	}
}

?>