<?php
## Database compatibility functions
# @author legolas558
#
# sanitizes SQL so that it is conformant to the format used by Drake
# contains hotfixes for Limbo's broken SQL

function	fix_sql($sql) {
	//1. identify the prefix used throughout the SQL file
	preg_match('/create *table *([^\\s]*categories) /i', $sql, $m);
	$p = strpos($m[1], 'categories');
	$prefix = strtolower(substr($m[1], 0, $p));

	//1.a: rename tables
	$sql = str_replace($prefix.'limbots', '#__drabots', $sql);

	//2. fix sql statements
	if ($prefix!='#__') {
		$sql = preg_replace('/create\\s+table\\s+'.$prefix.'/i', 'CREATE TABLE #__', $sql);
		$sql = preg_replace('/drop\\s+table\\s+'.$prefix.'/i', 'DROP TABLE #__', $sql);
		$sql = preg_replace('/insert\\s+into\\s+'.$prefix.'/i', 'INSERT INTO #__', $sql);
	}

/*
	function insert_alterator($m, $field, $nf, $nv) {
		if (!preg_match('/,\\s*'.$field.'/i', $m[1], $fi, PREG_OFFSET_CAPTURE))
			return $m[0];
		$left = substr($m[1], 0, $fi[0][1]);
		// get the offset of the field
		$ofs = preg_match_all('/,/', $left, $ignored);
		$right = substr($m[1], $fi[0][1] + strlen($fi[0][0]));
		preg_match_all('/,/', $m[2], $fio, PREG_OFFSET_CAPTURE);
		$values = substr($m[2], 0, $fio[0][$ofs][1]);
		if ($nf)
			$values .= $nf.', ';
		$values .= substr($m[2], $fio[0][$ofs+1][1]);
		// find position of definitions
		$dpos = strpos($m[0], $m[1]);
		// replace definitions
		$fixed = substr($m[0], 0, $dpos).$left.$nv.$right.substr($m[0], $dpos + strlen($m[1]));
		// find position of values
		$vpos = strpos($fixed, $m[2], $dpos);
		// replace values
		$fixed = substr($fixed, 0, $vpos).$values.substr($fixed, $vpos + strlen($m[2]));
		return $fixed;
	} */

	//2.a rename limbots to drabots
	function db_alterate($m) {
		return preg_replace("/'lim(\\w+)/", '\'dra$1', $m[0]);
	}

	$sql = preg_replace_callback('/insert\\s+into\\s+#__drabots\\s*\\(.*\\)\\s+values\\s*\\(.*\\)\\s*;/i', 'db_alterate', $sql);


	function alterator($m) {
		global $alt;
		$p = strpos($m[0], $alt[0]);
		if ($p===false) {
			if ($alt[2])
				return str_replace('PRIMARY KEY', $alt[0].' '.$alt[1]."\r\n".
					'PRIMARY KEY', $m[0]);
		}

		if (!$alt[2])
			return preg_replace('/\\s+'.$alt[0].'\\s+([^,]+,)/', ' '.$alt[1], $m[0]);
		return $m[0];
	}

	function alter_table(&$sql,$table, $field, $def, $new=true) {
		global $alt;
		$alt[0] = $field;
		$alt[1] = $def.',';
		$alt[2] = $new;

		$sql = preg_replace_callback('/create\\s+table\\s+#__'.$table.
		'\\s+([^;]*\\);)/i', 'alterator', $sql);
	}
	//3: add the new fields to the proper tables
	alter_table($sql, 'sections', 'editgroup', "int(3) NOT NULL DEFAULT '0'");
	// now the user changes browser but the CMS remembers his language
	alter_table($sql, 'users', 'lang', "varchar(2) NOT NULL DEFAULT 'en'");
//	add_field($sql, 'downloads', 'access', 'int(3) NOT NULL DEFAULT \'0\'');
	alter_table($sql, 'downloads', 'author', 'varchar(100) NOT NULL DEFAULT \'\'');
	alter_table($sql, 'downloads', 'image_url', 'varchar(255) NOT NULL DEFAULT \'\'');
	alter_table($sql, 'downloads', 'mod_date', 'varchar(20) NOT NULL DEFAULT \'\'');
	alter_table($sql, 'downloads', 'down_date', 'varchar(20) NOT NULL DEFAULT \'\'');

	//4: alter tables
	alter_table($sql, 'downloads', 'date', 'add_date varchar(20) NOT NULL DEFAULT \'\'', false);

	$oldfield = "/ip\\s+varchar\\(\\d+\\)/i";
	$newfield = 'ip varchar(36)';	// IPV6 does not scare us
	$sql = preg_replace($oldfield, $newfield, $sql);
	$oldfield = '/ip\\s+int\\(\\d+\\)/i';
	$sql = preg_replace($oldfield, $newfield, $sql);

//	$sql = str_replace("usertype varchar(25) NOT NULL DEFAULT '',", '', $sql);

	$sql = str_replace('message_subject text,', 'message_subject varchar(255),', $sql);

	//5: fix default declarations
	function def_reformat($m) {
		$s = str_replace( array('"', "'"), '', $m[2]);
		if (!$s)
			$s = 0;
		return 'int('.$m[1].') NOT NULL DEFAULT '.$s.',';
	}
	$sql = preg_replace_callback(
		'/int\\s*\\((\\d+)\\)\\s+NOT\\s+NULL\\s+DEFAULT\\s+(\'?0?\'?),/i',
				'def_reformat', $sql);

	//6: sanitize data from the old weird format
	// special characters will be handled at the db driver side
	$prev_sz = strlen($sql);
	function normalize($m) {
		$s = substr($m[0], 1, strlen($m[0])-2);
		if (!strlen($s)) return "''";

		$lev0 = str_replace(	array("[CR][NL]","[CR]","[NL]", "[ES][SQ]",
			"[ES][DQ]"),
							array("\n", "\n", "\n", "''", '"'), $s);
		$lev0 = str_replace( array('[SQ]', '[DQ]', '[ES]'), array("''", '"', '\\'), $lev0);
		return "'".
//		sql_encode(xhtml_encode($lev0, null), null)	// please check xhtml.php for proper filtering
		sql_encode($lev0, null)
		."'";
	}
	$sql = preg_replace_callback('/'.__QUOTED_S.'/i', 'normalize', $sql);

	//7: applies an Ms-Word cleanup filter to content items
	global $d_root;
	include $d_root.'includes/msword_clean.php';
	function msword_clean($m) {
		return $m[1].msw_clean($m[2])."', '".msw_clean($m[3])."'";
	}
	$sql = preg_replace_callback("/(content.*metadesc,access,hits\)[^V]*VALUES \\(\\d*+,'.*','.*',')(.*)','(.*?)'/", 'msword_clean', $sql);

//	echo 'Bytes gain: '.($prev_sz-strlen($sql));		// debug line

	//8: fix Limbo's SQL export bugs (missing fields)
	if (preg_match_all('/VALUES \\(.*?[\'\\d\\)]{1}(,,)[\'\\d\\)]{1}.*?\\);/', $sql, $m, PREG_OFFSET_CAPTURE)) {
		$ofs=0;
		foreach ($m[1] as $bug) {
			$sql = substr($sql, 0, $bug[1]+$ofs).
					",0,".						// patches ordering
					substr($sql, $bug[1]+2+$ofs);
			$ofs++;
		}
	}

	if (preg_match_all('/VALUES\\s+\\(.*?[\'\\d\\)]{1}(,\\));/', $sql, $m, PREG_OFFSET_CAPTURE)) {
		$ofs=0;
		foreach ($m[1] as $bug) {
			$sql = substr($sql, 0, $bug[1]+$ofs).
					",0".						// patches hits
					substr($sql, $bug[1]+2+$ofs);
			$ofs++;
		}
	}

/*//
	ob_end_clean();
	header('content-type: text/plain');
	echo($sql);
	die;	//*/

	return $sql;
}


?>
